
include "Spawner.lua"
include "Raid.lua"
include "RaidPresets.lua"

--from is an object, key is a static key, ie: fy_Crystal
local function FindClosestFromKey(from, key)

    return FindClosestFromTable(from, key.GetAll(), nil)

end


function OnNewLevelLoaded()
	fy_MagicMine.__Global.maxWorkers = 1
    local function NewCrystal( faction, position, forPlayer)
        local newCrystal = fy_Crystal.New(faction, position)
        newCrystal.isInvincible = true
        newCrystal.unitBasePrice = sv_Settings.baseBricktronPrice
        newCrystal.unitPriceMultiplier = sv_Settings.baseBricktronPriceMultiplier

        if forPlayer then
            newCrystal.unitsPossibleToSpawn = sv_Settings.startingFireflies
        end
        return newCrystal
    end

    local spawnPoints = Markers.GetGroup("Player Starting Point")

	local factionsToSpawn = {}
	for x, faction in pairs(Faction.GetAll()) do
		if faction.HasOwner() then table.insert(factionsToSpawn, faction) end
	end
	for i = 1, sv_Settings.startingCorruptCrystals do
		table.insert(factionsToSpawn, Faction.defaultEnemy)
	end
	
    Rules.playerRelations = sv_Settings.playerRelations
	Skybox.dayTime = sv_Settings.startingTimeOfDay	
	Skybox.dayLengthFactor = sv_Settings.daytimeFactor
	Skybox.nightLengthFactor = sv_Settings.nighttimeFactor
	
	if sv_Settings.daynightCycleSetting == 1 then
		Skybox.nightLengthFactor = 0.001
	elseif sv_Settings.daynightCycleSetting == 2 then
		Skybox.dayLengthFactor = 0.001
	end
	
    if #spawnPoints == 0 then
        error("No spawnpoints in map")
    else
		DetermineCrystals(factionsToSpawn)
		for x, spawn in pairs(spawnPoints) do
			
			local spawnCrystal = NewCrystal(spawn.desiredFaction, spawn.position, spawn.desiredFaction.HasOwner())
			spawnCrystal.SetAsSpawnCrystal()

            local closeMine = FindClosestFromKey(spawnCrystal, fy_MagicMine)
            if closeMine ~= nil then closeMine.maxWorkers = 2 end

			if spawn.desiredFaction.HasOwner() then
                for i = 1, sv_Settings.startingWorkersCount do
                    Spawner.Around(fy_Bricktron, spawn.position, spawn.desiredFaction, nil)
                end
                for i = 1, sv_Settings.startingKnightCount do
                    Spawner.Around(fy_Bricktron, spawn.position, spawn.desiredFaction, Occupations.Knight)
                end
                for i = 1, sv_Settings.startingArcherCount do
                    Spawner.Around(fy_Bricktron, spawn.position, spawn.desiredFaction, Occupations.Archer)
                end
			end
			
			 UpdatePopCap(spawn.desiredFaction,fy_Crystal.CountOfFaction(spawn.desiredFaction))
		end

        -- Spawn neutrals
		spawnPoints = Markers.GetGroup("Crystal")
        while #spawnPoints > 0 do
            spawn = table.remove(spawnPoints, math.random(#spawnPoints))
            NewCrystal(Faction.defaultNeutral, spawn.position, false).SetAsCaptureCrystal()
        end

    end

    --if not IsMultiplayer() then

        Registry.timers.raidBudget = Timer.New(sv_Settings.raidBudgetIncreaseInterval)
        Registry.timers.timeBeforeHostility = Timer.New(sv_Settings.minimumTimeBeforeHostilities)
        Registry.timers.maxTimeBetweenAttacks = Timer.New(sv_Settings.maximumTimeBetweenAttacks)
        Registry.timers.timeBetweenLevels = Timer.New(sv_Settings.enemyLevelUpInterval)
        Registry.timers.timeBeforeReinforcement = Timer.New(sv_Settings.reinforcementInterval)
    --end

	if IsMultiplayer() then
		StartTimers()
	end

			
		
		
end
Hooks.Connect(hk_OnNewLevelLoaded, OnNewLevelLoaded)

function UpdatePopCap(faction,crystalCount)
	if (IsServer()) then
		faction.SetMaximumPopulation(sv_Settings.bricktronPopulationCap + (crystalCount * sv_Settings.crystalPopulationIncrease))
	end
end

function FindClosestFromTable(from, table, predicate)

	predicate = predicate or |x,y| true
    local closestDistance = 9999.0
    local closestKey = nil

    for key, value in pairs(table) do

        local dist = Vector3.Distance(from.position, value.position)

        if dist < closestDistance and predicate(from, value) then

            closestDistance = dist
            closestKey = value
        end

    end

    return closestKey

end

function FindFurthestFromTable(from, table, predicate)
    
	predicate = predicate or function(x,y) return true end
    local closestDistance = 0.0
    local closestKey = nil

    for key, value in pairs(table) do

        local dist = Vector3.Distance(from.position, value.position)
        if dist > closestDistance and predicate(from, value) then
			closestDistance = dist
			closestKey = value
        end

    end

    return closestKey

end

--from is an object, key is a static key, ie: fy_Crystal
function FindFurthestFromKey(from, key)
    
    return FindFurthestFromTable(from, key.GetAll(), nil)

end

function SetFieldOfAll(table, fieldIndex, value)

	for x, element in pairs(table) do 
		element[fieldIndex] = value
	end

end

function DetermineCrystals(factionsToAssign)
	
	dc_spawnPoints = Markers.GetGroup("Player Starting Point")

	--SetFieldOfAll(dc_spawnPoints, "faction", Faction.defaultNeutral)
	for x, element in pairs(dc_spawnPoints) do 
		element.desiredFaction = Faction.defaultNeutral
	end
	for x, element in pairs(dc_spawnPoints) do 
		element.desiredValue = 0
	end
	--SetFieldOfAll(dc_spawnPoints, "desiredValue", 0)

	local stringoutput1 = ""
	for x, faction in pairs(factionsToAssign) do

		stringoutput1 = stringoutput1.."--"..x .. " " .. faction.tostring() .. "\n"
	end
	--print("-Going to assign\n"..stringoutput1)

	local factionsAssigned = 0

	for fi = 1, #factionsToAssign, 1 do
		
		local factionToAssign = factionsToAssign[fi]

		if factionsAssigned == 0 then
				
			dc_spawnPoints[ math.random(#dc_spawnPoints)].desiredFaction = factionToAssign
			factionsAssigned = factionsAssigned + 1

		else

			local infLoopPrev = 0
			local desisionMade = false
			while not desisionMade do

				--Go through every point
				for x, spawn1 in pairs(dc_spawnPoints) do 

					--If the base point is not neutral
					if spawn1.desiredFaction ~= Faction.defaultNeutral then

						local chosenSpawn = nil

						local predicate = function(spawn1, spawn2)
							return not spawn1.desiredFaction.IsEnemyTo(spawn2.desiredFaction) and spawn1.desiredFaction ~= spawn2.desiredFaction and spawn2.desiredFaction == Faction.defaultNeutral
						end

						if spawn1.desiredFaction.IsAlliedTo(factionToAssign) then
							chosenSpawn = FindClosestFromTable(spawn1, dc_spawnPoints, predicate)
						else
							chosenSpawn = FindFurthestFromTable(spawn1, dc_spawnPoints, predicate)
						end
						

						chosenSpawn.desiredValue = chosenSpawn.desiredValue + 1
						--print("-- "..spawn1.desiredFaction.tostring().." is considering "..chosenSpawn.tostring().. " at "..chosenSpawn.desiredValue.."/"..factionsAssigned)

						if chosenSpawn.desiredValue == factionsAssigned then
							--print("--- Chose "..chosenSpawn.tostring() .. " for " .. factionToAssign.tostring())
							chosenSpawn.desiredFaction = factionToAssign
							factionsAssigned = factionsAssigned + 1
							desisionMade = true
							break
						end

					end -- end of if neutral 

				end --end of for loop

				infLoopPrev = infLoopPrev + 1
				if infLoopPrev >= 128 then
					break
				end

			end --end of while loop

		end -- end of else factionAssigned > 0

	end -- end of for each factions

	--SetFieldOfAll(dc_spawnPoints, "desiredValue", 0)
	for x, element in pairs(dc_spawnPoints) do 
		element.desiredValue = desiredValue
	end

	
	local stringoutput = ""
	for x, spawn in pairs(dc_spawnPoints) do

		stringoutput = stringoutput.."--"..x .. " " .. spawn.tostring() .. spawn.desiredFaction.tostring() .. "\n"
	end
	--print("-Done!\n"..stringoutput)

end



function StartTimers()


    Registry.defendRaidRequired = sv_Settings.minimumSimultaneousDefenceRaid
    Registry.attackRaidLimit = sv_Settings.maximumSimultaneousAttackRaid

    for i = 1, sv_Settings.startingRaids do
        Raid.SpawnOnCrystal(0)
    end

    Registry.raidBudget = sv_Settings.startingRaids

    Registry.timers.raidBudget.Start()
    Registry.timers.timeBeforeHostility.Start()
    Registry.timers.maxTimeBetweenAttacks.Start()
    Registry.timers.timeBetweenLevels.Start()
    Registry.timers.timeBeforeReinforcement.Start()




end

function SlowUpdate()
    --if not IsMultiplayer() then
        if Registry.timers.raidBudget.Check() then
            if Registry.raidBudget == 0 then
                Registry.raidBudget = Registry.raidBudget + 1
            end
            Registry.timers.raidBudget.Reset()
        end

        if Registry.timers.timeBetweenLevels.Check() then
            Registry.currentLevel = Registry.currentLevel + 1
            Registry.timers.timeBetweenLevels.Reset()
        end

        if Registry.timers.timeBeforeReinforcement.Check() then
            if Registry.raidBudget > 0 and(Registry.attackRaidsCount + Registry.defendRaidsCount) < sv_Settings.maximumRaids then
                Raid.SpawnOnCrystal(0)
            else
                Registry.missedReinforcements = Registry.missedReinforcements + 1
            end

            if Registry.missedReinforcements >= sv_Settings.missedReinforcementsForSkirmish and Registry.skirmishRaids < sv_Settings.maximumSkirmishRaids then
                Raid.SpawnOnCrystal(1)
                Registry.missedReinforcements = 0
            end

            Registry.timers.timeBeforeReinforcement.Reset()
        end
    --end
    if IsBuildDebug() then
    	GameModeDebug()
    end
end
Hooks.Connect(hk_SlowUpdate, SlowUpdate)

function OnSerializing()

    --if not IsMultiplayer() then
        sv_SaveParams.Registry = Registry
    --end

end
Hooks.Connect(hk_OnSerializing, OnSerializing)

function OnDeserialized()

    --if not IsMultiplayer() then
        Registry = sv_SaveParams.Registry

    --end

end
Hooks.Connect(hk_OnDeserializing, OnDeserialized)


function GameModeDebug()
    GMDebugText.Text.Text = string.format(
[[<color=#ffc000ff>raidBudget</color> {0}
<color=#ffc000ff>currentLevel</color> {1}
<color=#ffc000ff>Raid count</color> {2} (<color=#ffc000ff>ATK</color> {3}  <color=#ffc000ff>SKR</color> {4}  <color=#ffc000ff>DEF</color> {5})
<color=#ffc000ff>raidBudget</color> {6:0} 
<color=#ffc000ff>timeBeforeHostilities</color> {7:0}
<color=#ffc000ff>maxTimeBetweenAttacks</color> {8:0}
<color=#ffc000ff>timeBetweenLevels</color> {9:0}
<color=#ffc000ff>timeBeforeReinforcement</color> {10:0}]],
    Registry.raidBudget, Registry.currentLevel, fy_RaidProject.CountOfFaction(CorruptedFaction), Registry.attackRaidsCount, Registry.skirmishRaids, Registry.defendRaidsCount, 
    Registry.timers.raidBudget.RemainingSeconds(),
    Registry.timers.timeBeforeHostility.RemainingSeconds(),
    Registry.timers.maxTimeBetweenAttacks.RemainingSeconds(),
    Registry.timers.timeBetweenLevels.RemainingSeconds(),
    Registry.timers.timeBeforeReinforcement.RemainingSeconds())
end
