local livingBaddies = false
gameOverDelay = Timer.New(2.0)

function InitInvasion()
    -- Skybox.DayPause = false

    local factions = Faction.GetAll()
    local spawnPoints = Markers.GetGroup("Player Starting Point")

    if #spawnPoints == 0 then
        error("No spawnpoints in map")
    else
        for x, faction in pairs(factions) do
            if faction.HasOwner() then
                local spawnPoint = spawnPoints[1]
                local spawnCrystal = fy_Crystal.New(faction, spawnPoint.position)
                spawnCrystal.unitsPossibleToSpawn = sv_Settings.startingFireflies
                spawnCrystal.unitBasePrice = sv_Settings.baseBricktronPrice
                spawnCrystal.unitPriceMultiplier = sv_Settings.baseBricktronPriceMultiplier

                for i = 1, sv_Settings.startingWorkersCount do
                    fy_Bricktron.New(faction, Random.VoxelAround(spawnPoint.position))
                end
                for i = 1, sv_Settings.startingKnightCount do
                    fy_Bricktron.New(faction, Random.VoxelAround(spawnPoint.position)).SetOccupation(Occupations.Knight)
                end
                for i = 1, sv_Settings.startingArcherCount do
                    fy_Bricktron.New(faction, Random.VoxelAround(spawnPoint.position)).SetOccupation(Occupations.Archer)
                end
            end
        end


    end

    Skybox.dayTime = sv_Settings.startingTimeOfDay
    Skybox.dayLengthFactor = sv_Settings.daytimeFactor
    Skybox.nightLengthFactor = sv_Settings.nighttimeFactor

    if sv_Settings.daynightCycleSetting == 1 then
        Skybox.nightLengthFactor = 0.0001
    elseif sv_Settings.daynightCycleSetting == 2 then
        Skybox.dayLengthFactor = 0.0001
    end

    Rules.canCaptureCapturables = false
    Rules.canSpawnMilitaryUnits = true
    Rules.maximumPopulation = sv_Settings.bricktronPopulationCap



end

function InvasionTrack()
    -- If the player is playing, so after he acknowledged the disclaimer
    if sv_SaveParams.waveTimer.running then
        -- Checks timer for new waves
        if sv_SaveParams.waveTimer.Check() then

            NextWave()

        elseif Math.FloorToInt(sv_SaveParams.waveTimer.RemainingSeconds()) == 30 then
            Sound.Play("Survival_Warning")
        end

        if livingBaddies then
            if fy_RaidProject.CountOfFaction(CorruptedFaction) == 0 then
                Sound.Play("Survival_Safe")
                livingBaddies = false
            end
        end
    end

    -- Calls gameover, need to bo coroutined
    if (gameOverDelay.Check()) then
        DisplayOkDialog("Game Over",
        "You survived <color=#ffc000ff>" .. sv_SaveParams.waveNumber .. "</color> wave" ..(sv_SaveParams.waveNumber > 1 and "s!" or "!"),
        "Return to title screen",
        function() Hooks.Invoke(hk_CallEndGame) end)
    end
end

-- Calls gameover, need to be coroutined
function PlayerLost(...)
	if (fy_User.localUser.alliedFaction ~= Faction.spectator )then
    gameOverDelay.Start()
	end
end

function SlowUpdate()
    InvasionTrack()
end
Hooks.Connect(hk_SlowUpdate, SlowUpdate)

function NextWave()
    GMtimer.Unparent()
    if not sv_SaveParams.waveTimer.running then
        sv_SaveParams.waveTimer.Start(sv_Settings.waveDuration + sv_Settings.firstWaveDurationBonus)
    else
        DoWaveSpawns()
        sv_SaveParams.waveTimer.Start(sv_Settings.waveDuration)
        sv_SaveParams.waveNumber = sv_SaveParams.waveNumber + 1
        if (not Sound.IsPlaying("Survival_New_Wave")) then
            Sound.Play("Music_Stop")
            Sound.Play("Survival_New_Wave")
        end
    end
    LUI.Scoreboard.Add(GMtimer)
	Table = {}
	Table[0] = sv_SaveParams.waveTimer
	Hooks.Invoke(hk_OnServerMessage,{Table})
end

function DoWaveSpawns()
	if IsServer() then
		local spawns = Markers.GetGroup("Corruptron Spawn Point")

		local wasAffordingRaid = true
		local budgetForWave = sv_SaveParams.waveBudget

		local raid
		local spawnPosition = spawns[math.random(#spawns)].position

		while wasAffordingRaid do

			local preset = RaidPresets.GetUnderBudget(budgetForWave)
			if preset == nil then

				wasAffordingRaid = false

			else

				if raid == nil then
					raid = fy_RaidProject.New(CorruptedFaction, Vector3.zero)
					raid.name = "Wave " .. tostring(sv_SaveParams.waveNumber)
				end

				budgetForWave = budgetForWave - RaidPresets.GetCostOf(preset)

				for x, pair in pairs(preset) do

					for y = 1, pair[1] do

						local newguy = pair[2].Ref.New(CorruptedFaction, Random.VoxelAround(spawnPosition))
						raid.AssignLabor(newguy)

					end
				end
			end
		end

		if raid ~= nil then
			raid.GiveMission(RaidMission.New(RaidMission.Type.Exterminate, fy_Crystal.GetFirst().position))
		end

		local budgetIncrease =(sv_Settings.additionalWaveBudget * sv_Settings.waveBudgetMultiplier) + Random.Range(- sv_Settings.waveBudgetRandom, sv_Settings.waveBudgetRandom)
		sv_SaveParams.waveBudget = sv_SaveParams.waveBudget + budgetIncrease
		livingBaddies = true
	end
end
