
local timerColorBlue = Color.New(0.0, 0.5, 1.0, 1.0)
local timerColorYellow = Color.yellow
local timerColorRed = Color.New(1.0, 0.0, 0.25, 1.0)
local timerColorLightRed = Color.New(1.0, 0.25, 0.25, 1.0)

function UpdateTimer()

    if sv_Settings.displayTimeBeforeWave == true then

        local remaining = sv_SaveParams.waveTimer.RemainingSeconds()
		
		if remaining < 1 then GMtimer.Text.Text = "0:00"
		else GMtimer.Text.Text = string.format("{0}:{1:00}", math.floor(remaining / 60.0), math.floor(remaining % 60.0)) end

        if sv_SaveParams.waveTimer.running then

            GMtimer.Text.Color =(remaining < 10 and timerColorRed or timerColorYellow)

        else

            GMtimer.Text.Color = timerColorBlue

        end

    else

        GMtimer.Text.Text = "(Hidden)"
        GMtimer.Text.Color = timerColorLightRed

    end
end

function LaborCreated(labor)

    if labor.faction == Faction.defaultAllied then

        GMBricktrons.OnAddLabor(labor)

    end
end

function Update()
    UpdateTimer()
end
Hooks.Connect(hk_Update, Update)

function LaborDeleted(labor)

    if labor.faction == Faction.defaultAllied then

        GMBricktrons.OnRemoveLabor(labor)

    end
end

function DisplayDisclaimer()

    DisplayOkDialog("Invasion", "invasion_description",
    "button_ready",
    function() sv_SaveParams.waveTimer.Start(sv_Settings.waveDuration + sv_Settings.firstWaveDurationBonus) end)
end

function OnNewRaidProject(raid)

	if raid.faction == CorruptedFaction then
		local wave = LUI.WaveStatus.New()
        wave.Raid = raid
        LUI.Scoreboard.Add(wave)
	end

end

function UISetup()

    if GMBricktrons == nil then
        -- Init ui
        GMBricktrons = LUI.Player.New()
        LUI.Scoreboard.Add(GMBricktrons)


        for x, raid in pairs(fy_RaidProject.GetAllOfFaction(CorruptedFaction)) do

            local wave = LUI.WaveStatus.New()
            wave.Raid = raid
            LUI.Scoreboard.Add(wave)

        end

        GMtimer = LUI.Timer.New()
        -- Building timer
        UpdateTimer()
        if sv_Settings.canManualTriggerWaves then
            GMtimer.SetNext( function() Picking.ResetCommandMode(); NextWave() end)
        end


        -- Building leaderboard
        LUI.Scoreboard.Add(GMtimer)
        if IsBuildDebug() then
            SpawnButton = LUI.Button.New()
            SpawnButton.Text.Text = "Test Spawn"
            SpawnButton.ButtonAction = function()
                local raid = fy_RaidProject.New(CorruptedFaction, Vector3.zero)
                raid.name = "New York City Boys"
                local spawnPosition = MainCamera.targetLocation
                local specialPreset = {
                    { 1, Characters.Biftron },
                    { 2, Characters.Minitron },
                    { 2, Characters.Corruptron }
                }
                for x, pair in pairs(specialPreset) do

                    for y = 1, pair[1] do

                        local newguy = pair[2].Ref.New(CorruptedFaction, Random.VoxelAround(spawnPosition))
                        raid.AssignLabor(newguy)

                    end
                end

                raid.GiveMission(RaidMission.New(RaidMission.Type.Exterminate, fy_Crystal.GetFirst().position))
                local wave = LUI.WaveStatus.New()
                wave.Raid = raid
                LUI.Scoreboard.Add(wave)

            end
            LUI.Scoreboard.Add(SpawnButton)
        end

        Hooks.Connect(hk_OnLaborCreated, LaborCreated)
        Hooks.Connect(hk_OnLaborDeleted, LaborDeleted)
		Hooks.Connect(fy_RaidProject.hk_OnNew, OnNewRaidProject)
        Sound.Play("Survival")
    end
end

